==> qtella-0.6.4.tar.gz: qtella-0.6.4/src/Servent.cpp <==
#include "../config.h"
#include "Gnutella.h"
#include "Servent.h"
#include "Connections.h"
#include "QtellaSub.h"
#include "Logging.h"

#include "Ping.h"
#include "Host.h"
#include "Address.h"
#include "Message.h"
#include "PongCache.h"
#include "Bye.h"
#include "StringManipulation.h"
#include "WidgetConfig.h"
#include "WidgetConnections.h"

#include <string>
#include <vector>
#include <sstream>

#include <qsocket.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qhostaddress.h>
#include <qlineedit.h>
#include <qlistbox.h>

#undef _DEBUG
#undef _SHOW_MESSAGES

#ifdef _SHOW_MESSAGES
#include <iostream>
#endif

Servent::Servent(Connections* parent, std::string id, QListViewItem *item, QSocket* socket, 
	const std::string user_agent, const std::string handshake)
	: _item(item), _timeout(0), _id(id), _queries(0), _state(Idle),
	_parent(parent), _timeout_counter(0), _bandwidth_counter(0),
	_ping_counter(0), _direction(0), _bytes_in(0), _bytes_out(0),
	_bandwidth_in(0), _bandwidth_out(0), _last_ping_send(0), _input(0),
	_output(0), _user_agent(user_agent), _header(handshake)
{
	_bye = 200;
	if( _user_agent.empty() )
		_version = "0.4";
	else
		_version = "0.6";

	_last_input = time(NULL);
	_start = time(NULL);
	_last_search = 0;

	if( socket ) 
		_socket = socket; 
	else 
		_socket = new QSocket();

	QObject::connect(_socket, SIGNAL(error(int)), SLOT(slotError(int)));
	QObject::connect(_socket, SIGNAL(connected()), SLOT(slotConnect()));
	QObject::connect(_socket, SIGNAL(readyRead()), SLOT(slotReadyRead()));
	QObject::connect(_socket, SIGNAL(connectionClosed()), SLOT(slotClosed()));
	QObject::connect(_socket, SIGNAL(delayedCloseFinished()), SLOT(slotDelayedClosed()));

	if( socket )
		if( _socket->state() == QSocket::Connection ) {
		if( _version == "0.4" )
			sendMessageDirect( QtellaSub::getInstance()->_gnutella_id + " OK\n\n" );
		Ping p(1, 0, _id);
		sendMessageDirect(p._data);  // send initial ping
		_parent->_statistic.out_ping_n++;
		_parent->_statistic.out_ping_b += p._data.size();
		setState(Connected);
	} else
		setState(Closed);

	_servant_number = Servent::getNextServentNumber();
	_timeout_timer = new QTimer(this);
	QObject::connect(_timeout_timer, SIGNAL(timeout()), SLOT(timeout()));
	_timeout_timer->start(1000);
}


Servent::~Servent()
{
  checkXTry();

  std::stringstream str;
  str << "Destructor called for " << _address.toString() << ". Servent: <" << _user_agent 
      << "> Buffer size: " << _buffer.size() << " Tx: " << _output
      << " Rx: " << _input;
  Logging::getInstance()->log( Logging::Info, str.str() );

  PongCache::getInstance()->removePong(_servant_number);
  if(_item) delete _item;
  if(_timeout_timer) delete _timeout_timer;

  QObject::disconnect(_socket, 0, 0, 0);
  if(_socket) 
    {
      Bye bye( _bye );
      sendMessageDirect( bye.toString() );

      delete _socket;
    }
}


void Servent::setByeCode( const int code )
{
  _bye = code;
}


const int Servent::getBufferSize()
{
  return _buffer.size();
}


int Servent::getNextServentNumber()
{
  static unsigned int number = 1;
  ++number;
  if(!number) number = 1;
  return number;
}


void Servent::connect(Address address)
{
  _address = address;
  _strip.erase();
  _strip = address.strIP();

  setState(Connecting);
  _timeout = 0;
  _socket->connectToHost(_strip.c_str(), address.port());
}



void Servent::slotDelayedClosed()
{
  setState(Closed);
}


void Servent::close()
{
  Bye bye( _bye );
  sendMessageDirect( bye.toString() );

  _socket->close();
  setState(Closed);
}



int Servent::state()
{
  return _state;
}



void Servent::setState(int state)
{
	if( (state == Closed) && (!_message_buffer.empty()) )
		_state = Closing;
	else
		_state = state;
}



void Servent::slotError(int err)
{
	#ifdef _DEBUG
		qDebug( "Serven::slotError" );
	#endif
	// check whether we've got the X-Try field
	checkXTry();
	slotReadyRead();
	std::stringstream str;
	str << "Connection error. Servent: <" << _user_agent 
		<< "> Buffer size: " << _buffer.size() << " Tx: " << _output 
		<< " Rx: " << _input;
	Logging::getInstance()->log( Logging::Error, str.str() );
	if( _socket->state() != QSocket::Idle ) 
		_socket->close();
	setState(Error);
	_timeout = 0;
}



void Servent::writeConnectString()
{
	QtellaSub* qs = QtellaSub::getInstance();
	_timeout = 0;
	std::string connect_string;
	if( 0 ) {
		connect_string = std::string (qs->_gnutella_id + " CONNECT/0.4\n\n");
		_version = "0.4";
	} else {
		QHostAddress ha = _socket->peerAddress();
		connect_string  = qs->_gnutella_cid + " CONNECT/0.6\r\n";
		if( qs->_lan_mode ) {
			connect_string += "LAN: " + qs->_gnutella_id + "\r\n";
		}
		connect_string += "User-Agent: Qtella " 
			+ std::string(VERSION) + ADDITIONAL_VERSION_STR + "\r\n"
			+ "Remote-IP: " + std::string( ha.toString().latin1() ) + "\r\n"
//			+ "X-Qtella-User: " + std::string( _parent->_parent->_widget_config->ui_lineedit_userid->
//				text().latin1() ) + "\r\n" 
//			+ "X-Qtella-Group: " + std::string( _parent->_parent->
//				_widget_config->ui_lineedit_groupid->text().latin1() ) + "\r\n"
			#ifdef _DEFLATE
      			+ "Accept-Encoding: deflate\r\n"
			#endif
			+ "\r\n";
			_version = "0.6";
	}
	sendMessageDirect(connect_string);
}



void Servent::slotConnect()
{
	setState(Waiting);
	writeConnectString();
}



const void Servent::checkXTry()
{
#ifdef _DEBUG
  qDebug( "Servent::checkXTry()" );
#endif
  if (_buffer.find ("X-Try:") == std::string::npos)
    return;

  // great, we've got IPs of other hosts
  std::string s;
  copy( _buffer.begin(), _buffer.end(), back_inserter( s ) );

#ifdef _DEBUG
  qDebug( "Got: <%s>", s.c_str() );
#endif

  std::string::size_type pos = s.find( "X-Try:" );
  s = s.substr( pos + 6 );
  pos = s.find_first_of( "0123456789" );
  std::string::size_type pos2 = s.find_first_of( "\n" );

  if( pos2 <= pos )
    return;

  s = s.substr( pos, pos2 - pos );
  s = s.substr( 0, s.find_last_of( "0123456789" ) + 1 );

#ifdef _DEBUG
  qDebug( "X-Try check: <%s>", s.c_str() );
#endif

  StringManipulation sm( s );
  std::vector< std::string > v;
  sm.split_to_vector( v, ',' );
  for (unsigned i = 0; i < v.size(); ++i)
	Connections::getInstance()->addHost( v[ i ] );
    //_parent->_parent->_widget_connections->ui_listbox_hostlist->insertItem( v[i].c_str(), -1);
}


void Servent::slotClosed()
{
#ifdef _DEBUG
  qDebug( "entering Servent::slotClosed(); my number is %d", _servant_number );
#endif

  // check whether we've got the X-Try field
  checkXTry();

  slotReadyRead();

  std::stringstream str;
  str << "Connection closed by foreign host. Servent: <" << _user_agent 
      << "> Buffer size: " << _buffer.size() << " Tx: " << _output
      << " Rx: " << _input;
  Logging::getInstance()->log( Logging::Info, str.str() );

  setState(Closed);
}



void Servent::sendMessageDirect( const std::string& buffer )
{
  if(_socket->state() == QSocket::Connection)
    {
      _output++;
      _bytes_out += buffer.size();
      _socket->writeBlock(buffer.data(), buffer.size());
      _socket->flush();
    }
}



void Servent::sendMessage(const std::string& buffer, bool cache)
{
  if( ((_send_buffer.size() < 512) && (_socket->bytesToWrite() < 1024))|| !cache )
    {
      _output++;
      _send_buffer.append(buffer);
    }
}


void Servent::limitVectors()
{
  while(_id_set.size() > 2000) _id_set.erase(_id_set.begin());
}


int Servent::getDirection()
{
  return _direction;
}


void Servent::setDirection(int i)
{
  _direction = i;
}

void Servent::slotReadyRead()
{
	char cbuff[4096+10];

	if( ! _socket->bytesAvailable() ) {
		return;
	}
	#ifdef SDEBUG
		qDebug( "Servent %d bytes available.", _socket->bytesAvailable() );
	#endif
	_last_input = time(NULL);
	int bytes = 0;
	do {
		int rd = _socket->readBlock( cbuff, 4096 );
		#ifdef SDEBUG
			qDebug( "Servent got message: (%s)", cbuff );
		#endif
		bytes += rd;
		_buffer.append( cbuff, rd );
	} while( _socket->bytesAvailable() );

	#ifdef SDEBUG
		qDebug( "Servent (%d) read %d bytes", _servant_number, bytes );
	#endif

	QtellaSub *qs = QtellaSub::getInstance();
	if( state() != Waiting ) {
    	while( _buffer.size() >= HEADER_SIZE ) {
			Q_UINT32 payload  = Message::str_uint32( _buffer.substr( 19, 4 ) );  // position 19, 4 bytes
			if( payload > 20000 ) {
				_buffer.erase();
				#ifdef SDEBUG
					qDebug( "Servent error (payload is big)" );
				#endif
				slotError( -1 );
				return;
        	}
			if( _buffer.size() >= HEADER_SIZE + payload ) {
				++_input;
				_parent->incAllPackets();
				unsigned limit = 40;
				if( time(NULL) - _last_search < 30 ) // if last search is not older than 30s 
					limit = 120; // set limit to 120 messages/s
				if( _message_buffer.size() < limit ) {
					std::vector< char > v;
					for( unsigned k = 0; k < HEADER_SIZE + payload; ++k )
						v.push_back( _buffer[k] );
					_message_buffer.push_back( v );
				} else
					_parent->incIgnoredPackets();
				#ifdef SDEBUG
					qDebug( "Servent removing message from buffer." );
				#endif
				std::string::iterator pos = _buffer.begin();
				for( unsigned i = 0; i < HEADER_SIZE + payload; ++i )
					++pos;
				_buffer.erase( _buffer.begin(), pos );
				_bytes_in += HEADER_SIZE + payload;
			} else
				return;
		} 
	} else if( _version == "0.6" && _buffer.find ( "\r\n\r\n" ) != std::string::npos &&
		_buffer.substr( 0, qs->_gnutella_cid.size () + 4) == ( qs->_gnutella_cid + "/0.6" ) ) {
		checkXTry();
		_header = _buffer.substr( 0, _buffer.find ( "\r\n\r\n" ) + 4 );
		#ifdef SDEBUG
			qDebug( ">>>>>header (%s) (%s) header<<<<<<", _header.c_str(), _buffer.c_str() );
		#endif
		_buffer.erase( 0, _header.size() );
		// get status code
		std::string status = _header.substr( _header.find( " " ) + 1, 3 );
      
		if( status == "503" ) {
			#ifdef SDEBUG
				qDebug( "remote servent full" );
			#endif
			slotError( -1 );
			return;
		}
      
		if( status != "200" ) {
			#ifdef SDEBUG
				qDebug( "Servent: unknown status <%s>, error", status.c_str() );
			#endif
			std::stringstream str;
			str << _address.toString() << " Status: <" << status << ">" << " Servent: <" << _user_agent 
				<< "> Buffer size: " << _header.size() << " Tx: " << _output
				<< " Rx: " << _input;
			Logging::getInstance()->log( Logging::Error, str.str() );
			slotError( -1 );
			return;
		}
		// status code is 200 ok
		// get Remote-IP field
		std::string ip = getHeaderField (_header, "Remote-IP");
		if( ip != "" ) {
			qs->setReportedIP( ip );
		}
		// get user agent information
		_user_agent = getHeaderField( _header, "User-Agent" );
		if( _user_agent == "" )
			_user_agent = "?";
		setState(Connected);
		sendMessageDirect (qs->_gnutella_cid + "/0.6 200 OK\r\n\r\n");
      
		Ping p(1, 0, _id); // for initial ping, use ttl = 1
		sendMessageDirect(p._data);  // send initial ping
		_parent->_statistic.out_ping_n++;
		_parent->_statistic.out_ping_b += p._data.size();
		return;
	} else if( _version == "0.4" && _buffer.substr( 0, qs->_gnutella_id.size () + 5 ) == 
		( qs->_gnutella_id + " OK\n\n" ) ) {
			_buffer.erase( 0, qs->_gnutella_id.size() + 5 );
			setState( Connected );
			Ping p( 1, 0, _id );
			sendMessageDirect( p._data );  // send initial ping
			_parent->_statistic.out_ping_n++;
			_parent->_statistic.out_ping_b += p._data.size();
			return;
	}
}


void Servent::timeout()
{
  _timeout_counter++;

  if(_timeout_counter >= 5)
    if( (time(NULL) - _last_input) >= 120 ) 
      {
        close();
        _timeout_counter = 0;
      }


  if( (_socket->state() == QSocket::Connection) && (state() == Connected) &&
      (_send_buffer.size() > 0) )
    {
      _bytes_out += _send_buffer.size();
      _socket->writeBlock(_send_buffer.data(), _send_buffer.size());
      _send_buffer.erase();
    }


  _bandwidth_counter++;

  if(_bandwidth_counter >= 2)
    {
      _bandwidth_in = static_cast<double>(_bytes_in) / static_cast<double>(2048);
      _bandwidth_out = static_cast<double>(_bytes_out) / static_cast<double>(2048);
      _bytes_in = 0;
      _bytes_out = 0;
      _bandwidth_counter = 0;
    }


  _ping_counter++;
  
  if(_ping_counter >= 3)
    {
      if( !_last_accepted_ping_id.empty() )
        if( ((static_cast<unsigned char>(_last_accepted_ping_id[8]) == 0xff) && 
             (_last_accepted_ping_id[15] == 1)) ||
            ((time(NULL) - _last_ping_send >= 30) && (state() == Connected)) )
          {
            int ttl = _parent->_parent->_widget_config->ui_spinbox_ttl->value();
            Ping p(ttl, 0, _id);
            sendMessage(p._data, false);
			
            _parent->_statistic.out_ping_n++;
            _parent->_statistic.out_ping_b += p._data.size();
			
            _last_ping_send = time(NULL);
          }
      _ping_counter = 0;
    }
  
  
  _queries = 0;
}


long Servent::uptime()
{
  return (time(NULL) - _start);
}


const void Servent::updateLastSearch()
{
  _last_search = time(NULL);
}

std::vector< char >& Servent::getFirstMessage()
{
	return _message_buffer.front();
}

const void Servent::removeFirstMessage()
{
	_message_buffer.pop_front();
}

const std::string& Servent::getUserID()
{
	return _user_id;
}

const std::string& Servent::getGroupID()
{
	return _group_id;
}

const std::string Servent::getHeaderField( const std::string& header, const std::string& field )
{
  std::string::size_type pos = header.find("\r\n" + field + ":" );
  if( pos == std::string::npos )
    return "";
  pos += field.size() + 3;
  std::string::size_type pos2 = header.find( "\r\n", pos );
  std::string s = header.substr( pos, pos2 - pos );
  s = s.substr( s.find_first_not_of( " " ) );
  while( ! s.empty() && s[s.size() - 1] == ' ' )
    s.erase( s.size() - 1 );
  return s;
}

==> qtella-0.6.5.tar.gz: qtella-0.6.5/src/Servent.cpp <==
#include "../config.h"
#include "Gnutella.h"
#include "Servent.h"
#include "Connections.h"
#include "QtellaSub.h"
#include "Logging.h"

#include "Ping.h"
#include "Host.h"
#include "Address.h"
#include "Message.h"
#include "PongCache.h"
#include "Bye.h"
#include "StringManipulation.h"
#include "WidgetConfig.h"
#include "WidgetConnections.h"

#include <string>
#include <vector>
#include <sstream>

#include <qsocket.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qhostaddress.h>
#include <qlineedit.h>
#include <qlistbox.h>

#undef _DEBUG
#undef _SHOW_MESSAGES

#ifdef _SHOW_MESSAGES
#include <iostream>
#endif

Servent::Servent(Connections* parent, std::string id, QListViewItem *item, QSocket* socket, 
	const std::string user_agent, const std::string handshake)
	: _item(item), _timeout(0), _id(id), _queries(0), _state(Idle),
	_parent(parent), _timeout_counter(0), _bandwidth_counter(0),
	_ping_counter(0), _direction(0), _bytes_in(0), _bytes_out(0),
	_bandwidth_in(0), _bandwidth_out(0), _last_ping_send(0), _input(0),
	_output(0), _user_agent(user_agent), _header(handshake)
{
	#ifdef _DEBUG
		qDebug( "Servent::Servent(): entering" );
	#endif
	_bye = 200;
	if( _user_agent.empty() )
		_version = "0.4";
	else
		_version = "0.6";

	_last_input = time(NULL);
	_start = time(NULL);
	_last_search = 0;

	if( socket ) 
		_socket = socket; 
	else 
		_socket = new QSocket();

	QObject::connect(_socket, SIGNAL(error(int)), SLOT(slotError(int)));
	QObject::connect(_socket, SIGNAL(connected()), SLOT(slotConnect()));
	QObject::connect(_socket, SIGNAL(readyRead()), SLOT(slotReadyRead()));
	QObject::connect(_socket, SIGNAL(connectionClosed()), SLOT(slotClosed()));
	QObject::connect(_socket, SIGNAL(delayedCloseFinished()), SLOT(slotDelayedClosed()));

	if( socket )
		if( _socket->state() == QSocket::Connection ) {
		if( _version == "0.4" )
			sendMessageDirect( QtellaSub::getInstance()->_gnutella_id + " OK\n\n" );
		Ping p(1, 0, _id);
		sendMessageDirect(p._data);  // send initial ping
		_parent->_statistic.out_ping_n++;
		_parent->_statistic.out_ping_b += p._data.size();
		setState(Connected);
	} else
		setState(Closed);

	_servant_number = Servent::getNextServentNumber();
	_timeout_timer = new QTimer(this);
	QObject::connect(_timeout_timer, SIGNAL(timeout()), SLOT(timeout()));
	_timeout_timer->start(1000);
	
	#ifdef _DEBUG
		qDebug( "Servent::Servent(): quit" );
	#endif
}


Servent::~Servent()
{
	#ifdef _DEBUG
		qDebug( "Servent::~Servent(): entering" );
	#endif
	checkXTry();
	std::stringstream str;
	str << "Destructor called for " << _address.toString() << ". Servent: <" 
		<< _user_agent << "> Buffer size: " << _buffer.size() << " Tx: " 
		<< _output << " Rx: " << _input;
	Logging::getInstance()->log( Logging::Info, str.str() );

	PongCache::getInstance()->removePong( _servant_number );
	if( _item ) delete _item;
	if( _timeout_timer ) delete _timeout_timer;

	QObject::disconnect(_socket, 0, 0, 0);
	if( _socket ) {
		Bye bye( _bye );
		sendMessageDirect( bye.toString() );
		_socket->deleteLater();
	}
	#ifdef _DEBUG
		qDebug( "Servent::~Servent(): quit" );
	#endif
}


void Servent::setByeCode( const int code )
{
  _bye = code;
}


const int Servent::getBufferSize()
{
  return _buffer.size();
}


int Servent::getNextServentNumber()
{
  static unsigned int number = 1;
  ++number;
  if(!number) number = 1;
  return number;
}


void Servent::connect(Address address)
{
	#ifdef _DEBUG
		qDebug( "Servent::connect(): entering" );
	#endif
	_address = address;
	_strip.erase();
	_strip = address.strIP();

	setState(Connecting);
	_timeout = 0;
	_socket->connectToHost(_strip.c_str(), address.port());
	#ifdef _DEBUG
		qDebug( "Servent::connect(): quit" );
	#endif
}



void Servent::slotDelayedClosed()
{
  setState( Closed );
}


void Servent::close()
{
	#ifdef _DEBUG
		qDebug( "Servent::close(): entering" );
	#endif
	Bye bye( _bye );
	sendMessageDirect( bye.toString() );
	_socket->close();
	setState(Closed);
	#ifdef _DEBUG
		qDebug( "Servent::close(): quit" );
	#endif
}



int Servent::state()
{
  return _state;
}



void Servent::setState(int state)
{
	if( (state == Closed) && (!_message_buffer.empty()) )
		_state = Closing;
	else
		_state = state;
}



void Servent::slotError( int err )
{
	#ifdef _DEBUG
		qDebug( "Servent::slotError(): entering" );
	#endif
	// check whether we've got the X-Try field
	checkXTry();
	slotReadyRead();
	std::stringstream str;
	str << "Connection error. Servent: <" << _user_agent 
		<< "> Buffer size: " << _buffer.size() << " Tx: " << _output 
		<< " Rx: " << _input;
	Logging::getInstance()->log( Logging::Error, str.str() );
	if( _socket->state() != QSocket::Idle ) 
		_socket->close();
	setState( Error );
	_timeout = 0;
	#ifdef _DEBUG
		qDebug( "Servent::slotError(): quit" );
	#endif
}



void Servent::writeConnectString()
{
	#ifdef _DEBUG
		qDebug( "Servent::writeConnectString(): entering" );
	#endif
	QtellaSub* qs = QtellaSub::getInstance();
	_timeout = 0;
	std::string connect_string;
	if( 0 ) {
		connect_string = std::string (qs->_gnutella_id + " CONNECT/0.4\n\n");
		_version = "0.4";
	} else {
		QHostAddress ha = _socket->peerAddress();
		connect_string  = qs->_gnutella_cid + " CONNECT/0.6\r\n";
		if( qs->_lan_mode ) {
			connect_string += "LAN: " + qs->_gnutella_id + "\r\n";
		}
		connect_string += "User-Agent: Qtella " 
			+ std::string(VERSION) + ADDITIONAL_VERSION_STR + "\r\n"
			+ "Remote-IP: " + std::string( ha.toString().latin1() ) + "\r\n"
//			+ "X-Qtella-User: " + std::string( _parent->_parent->_widget_config->ui_lineedit_userid->
//				text().latin1() ) + "\r\n" 
//			+ "X-Qtella-Group: " + std::string( _parent->_parent->
//				_widget_config->ui_lineedit_groupid->text().latin1() ) + "\r\n"
			#ifdef _DEFLATE
      			+ "Accept-Encoding: deflate\r\n"
			#endif
			+ "\r\n";
			_version = "0.6";
	}
	sendMessageDirect(connect_string);
	#ifdef _DEBUG
		qDebug( "Servent::writeConnectString(): quit" );
	#endif
}



void Servent::slotConnect()
{
	setState( Waiting );
	writeConnectString();
}



const void Servent::checkXTry()
{
	#ifdef _DEBUG
		qDebug( "Servent::checkXTry(): entering" );
	#endif
	if ( _buffer.find( "X-Try:" ) == std::string::npos )
		return;

	// great, we've got IPs of other hosts
	std::string s;
	copy( _buffer.begin(), _buffer.end(), back_inserter( s ) );

	#ifdef _DEBUG
		qDebug( "Servent::checkXTry(): got: <%s>", s.c_str() );
	#endif

	std::string::size_type pos = s.find( "X-Try:" );
	s = s.substr( pos + 6 );
	pos = s.find_first_of( "0123456789" );
	std::string::size_type pos2 = s.find_first_of( "\n" );

	if( pos2 <= pos )
		return;
	s = s.substr( pos, pos2 - pos );
	s = s.substr( 0, s.find_last_of( "0123456789" ) + 1 );
	#ifdef _DEBUG
		qDebug( "X-Try check: <%s>", s.c_str() );
	#endif
	StringManipulation sm( s );
	std::vector< std::string > v;
	sm.split_to_vector( v, ',' );
	for (unsigned i = 0; i < v.size(); ++i)
		Connections::getInstance()->addHost( v[ i ] );
    //_parent->_parent->_widget_connections->ui_listbox_hostlist->insertItem( v[i].c_str(), -1);
	#ifdef _DEBUG
		qDebug( "Servent::checkXTry(): quit" );
	#endif
}


void Servent::slotClosed()
{
	#ifdef _DEBUG
		qDebug( "Servent::slotClosed(): entering my number is %d", _servant_number );
	#endif
	// check whether we've got the X-Try field
	checkXTry();
	slotReadyRead();
	std::stringstream str;
	str << "Connection closed by foreign host. Servent: <" << _user_agent 
		<< "> Buffer size: " << _buffer.size() << " Tx: " << _output
		<< " Rx: " << _input;
	Logging::getInstance()->log( Logging::Info, str.str() );
	setState(Closed);
	#ifdef _DEBUG
		qDebug( "Servent::slotClosed(): quit" );
	#endif
}



void Servent::sendMessageDirect( const std::string& buffer )
{
  if(_socket->state() == QSocket::Connection)
    {
      _output++;
      _bytes_out += buffer.size();
      _socket->writeBlock(buffer.data(), buffer.size());
      _socket->flush();
    }
}



void Servent::sendMessage(const std::string& buffer, bool cache)
{
  if( ((_send_buffer.size() < 512) && (_socket->bytesToWrite() < 1024))|| !cache )
    {
      _output++;
      _send_buffer.append(buffer);
    }
}


void Servent::limitVectors()
{
  while(_id_set.size() > 2000) _id_set.erase(_id_set.begin());
}


int Servent::getDirection()
{
  return _direction;
}


void Servent::setDirection(int i)
{
  _direction = i;
}

void Servent::slotReadyRead()
{
	char cbuff[4096+10];
	if( ! _socket->bytesAvailable() ) {
		return;
	}
	#ifdef SDEBUG
		qDebug( "Servent %d bytes available.", _socket->bytesAvailable() );
	#endif
	_last_input = time(NULL);
	int bytes = 0;
	do {
		int rd = _socket->readBlock( cbuff, 4096 );
		#ifdef SDEBUG
			qDebug( "Servent got message: (%s)", cbuff );
		#endif
		bytes += rd;
		_buffer.append( cbuff, rd );
	} while( _socket->bytesAvailable() );

	#ifdef SDEBUG
		qDebug( "Servent (%d) read %d bytes", _servant_number, bytes );
	#endif

	QtellaSub *qs = QtellaSub::getInstance();
	if( state() != Waiting ) {
    	while( _buffer.size() >= HEADER_SIZE ) {
			// position 19,4 bytes
			Q_UINT32 payload  = Message::str_uint32( _buffer.substr( 19, 4 ) );
			if( payload > 20000 ) {
				_buffer.erase();
				#ifdef SDEBUG
					qDebug( "Servent error (payload is big)" );
				#endif
				slotError( -1 );
				return;
        	}
			if( _buffer.size() >= HEADER_SIZE + payload ) {
				++_input;
				_parent->incAllPackets();
				unsigned limit = 40;
				if( time(NULL) - _last_search < 30 ) // if last search is not older than 30s 
					limit = 120; // set limit to 120 messages/s
				if( _message_buffer.size() < limit ) {
					std::vector< char > v;
					for( unsigned k = 0; k < HEADER_SIZE + payload; ++k )
						v.push_back( _buffer[k] );
					_message_buffer.push_back( v );
				} else
					_parent->incIgnoredPackets();
				#ifdef SDEBUG
					qDebug( "Servent removing message from buffer." );
				#endif
				std::string::iterator pos = _buffer.begin();
				for( unsigned i = 0; i < HEADER_SIZE + payload; ++i )
					++pos;
				_buffer.erase( _buffer.begin(), pos );
				_bytes_in += HEADER_SIZE + payload;
			} else
				return;
		} 
	} else if( _version == "0.6" && _buffer.find ( "\r\n\r\n" ) != std::string::npos &&
		_buffer.substr( 0, qs->_gnutella_cid.size () + 4) == ( qs->_gnutella_cid + "/0.6" ) ) {
		checkXTry();
		_header = _buffer.substr( 0, _buffer.find ( "\r\n\r\n" ) + 4 );
		#ifdef SDEBUG
			qDebug( ">>>>>header (%s) (%s) header<<<<<<", _header.c_str(), _buffer.c_str() );
		#endif
		_buffer.erase( 0, _header.size() );
		// get status code
		std::string status = _header.substr( _header.find( " " ) + 1, 3 );
      
		if( status == "503" ) {
			#ifdef SDEBUG
				qDebug( "remote servent full" );
			#endif
			slotError( -1 );
			return;
		}
      
		if( status != "200" ) {
			#ifdef SDEBUG
				qDebug( "Servent: unknown status <%s>, error", status.c_str() );
			#endif
			std::stringstream str;
			str << _address.toString() << " Status: <" << status << ">" << " Servent: <" << _user_agent 
				<< "> Buffer size: " << _header.size() << " Tx: " << _output
				<< " Rx: " << _input;
			Logging::getInstance()->log( Logging::Error, str.str() );
			slotError( -1 );
			return;
		}
		// status code is 200 ok
		// get Remote-IP field
		std::string ip = getHeaderField (_header, "Remote-IP");
		if( ip != "" ) {
			qs->setReportedIP( ip );
		}
		// get user agent information
		_user_agent = getHeaderField( _header, "User-Agent" );
		if( _user_agent == "" )
			_user_agent = "?";
		setState(Connected);
		sendMessageDirect (qs->_gnutella_cid + "/0.6 200 OK\r\n\r\n");
      
		Ping p(1, 0, _id); // for initial ping, use ttl = 1
		sendMessageDirect(p._data);  // send initial ping
		_parent->_statistic.out_ping_n++;
		_parent->_statistic.out_ping_b += p._data.size();
		return;
	} else if( _version == "0.4" && _buffer.substr( 0, qs->_gnutella_id.size () + 5 ) == 
		( qs->_gnutella_id + " OK\n\n" ) ) {
			_buffer.erase( 0, qs->_gnutella_id.size() + 5 );
			setState( Connected );
			Ping p( 1, 0, _id );
			sendMessageDirect( p._data );  // send initial ping
			_parent->_statistic.out_ping_n++;
			_parent->_statistic.out_ping_b += p._data.size();
			return;
	}
}


void Servent::timeout()
{
  _timeout_counter++;

  if(_timeout_counter >= 5)
    if( (time(NULL) - _last_input) >= 120 ) 
      {
        close();
        _timeout_counter = 0;
      }


  if( (_socket->state() == QSocket::Connection) && (state() == Connected) &&
      (_send_buffer.size() > 0) )
    {
      _bytes_out += _send_buffer.size();
      _socket->writeBlock(_send_buffer.data(), _send_buffer.size());
      _send_buffer.erase();
    }


  _bandwidth_counter++;

  if(_bandwidth_counter >= 2)
    {
      _bandwidth_in = static_cast<double>(_bytes_in) / static_cast<double>(2048);
      _bandwidth_out = static_cast<double>(_bytes_out) / static_cast<double>(2048);
      _bytes_in = 0;
      _bytes_out = 0;
      _bandwidth_counter = 0;
    }


  _ping_counter++;
  
  if(_ping_counter >= 3)
    {
      if( !_last_accepted_ping_id.empty() )
        if( ((static_cast<unsigned char>(_last_accepted_ping_id[8]) == 0xff) && 
             (_last_accepted_ping_id[15] == 1)) ||
            ((time(NULL) - _last_ping_send >= 30) && (state() == Connected)) )
          {
            int ttl = _parent->_parent->_widget_config->ui_spinbox_ttl->value();
            Ping p(ttl, 0, _id);
            sendMessage(p._data, false);
			
            _parent->_statistic.out_ping_n++;
            _parent->_statistic.out_ping_b += p._data.size();
			
            _last_ping_send = time(NULL);
          }
      _ping_counter = 0;
    }
  
  
  _queries = 0;
}


long Servent::uptime()
{
  return (time(NULL) - _start);
}


const void Servent::updateLastSearch()
{
  _last_search = time(NULL);
}

std::vector< char >& Servent::getFirstMessage()
{
	return _message_buffer.front();
}

const void Servent::removeFirstMessage()
{
	_message_buffer.pop_front();
}

const std::string& Servent::getUserID()
{
	return _user_id;
}

const std::string& Servent::getGroupID()
{
	return _group_id;
}

const std::string Servent::getHeaderField( const std::string& header, const std::string& field )
{
	std::string::size_type pos = header.find("\r\n" + field + ":" );
	if( pos == std::string::npos )
		return "";
	pos += field.size() + 3;
	std::string::size_type pos2 = header.find( "\r\n", pos );
	std::string s = header.substr( pos, pos2 - pos );
	s = s.substr( s.find_first_not_of( " " ) );
	while( ! s.empty() && s[s.size() - 1] == ' ' )
		s.erase( s.size() - 1 );
	return s;
}
